package com.bd.framework.accessimpl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;

import com.bd.framework.util.Command;
import com.bd.framework.util.HibernateUtil;

/**
 * Base class for Access Objects.
 * It is responsible for some common 
 * Hibernate stuff and error handling.
 * <p>
 * Subclasses must implement {@link #performExecute()}
 * 
 * @author patrik.nordwall
 */
public abstract class AccessBase implements Command {
    
    private static final Log LOG = LogFactory.getLog(AccessBase.class);
    
    public void execute() {
        Transaction tx = null;
        try {
            Session session = HibernateUtil.currentSession();
            tx = session.beginTransaction();
            performExecute();
            tx.commit();
        } catch (HibernateException e) {
            LOG.error(e.getMessage(), e);
            if (tx != null) {
                try {
                    tx.rollback();
                } catch (HibernateException ignore) {
                    // not much to do
                }
            }
            throw new RuntimeException(e.getMessage());
        } catch (RuntimeException e) {
            LOG.error(e.getMessage(), e);
            throw e;
        }
    }
    
    public abstract void performExecute() throws HibernateException;
    
       

}
