
package com.bd.framework.accessimpl;

import java.io.Serializable;

import org.hibernate.HibernateException;
import org.hibernate.LockMode;
import org.hibernate.Session;

import com.bd.framework.accessapi.FindByIdAccess;
import com.bd.framework.util.HibernateUtil;

/**
 * <p>
 * Find an entity by its id.
 * Implementation of Access command FindByIdAccess. 
 * </p>
 * <p>
 * Command design pattern.
 * </p>
 */
public class FindByIdAccessImpl<T, ID extends Serializable> extends AccessBase 
    implements FindByIdAccess<T, ID> {

    private Class<T> persistentClass;
    
    private boolean lock;
    private ID id;
    private T result;
    
    public FindByIdAccessImpl(Class<T> persistentClass) {
        this.persistentClass = persistentClass;
    }
    
    protected Class<T> getPersistentClass() {
        return persistentClass;
    }

    public void setLock(boolean lock) {
        this.lock = lock;
    }

    public void setId(ID id) {
        this.id = id;
    }

    public T getResult() {
        return result;
    }
    
    @SuppressWarnings("unchecked")
    public void performExecute() throws HibernateException {
        Session session = HibernateUtil.currentSession();
        if (lock) {
            result = (T) session.load(getPersistentClass(), id, LockMode.UPGRADE);
        } else {
            result = (T) session.load(getPersistentClass(), id);
        }
    }
    

} 