package com.bd.framework.util;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;

/**
 * Utility for Hibernate stuff, such as session management.
 * 
 * @author patrik.nordwall
 */
public class HibernateUtil {
    private static final Log LOG = LogFactory.getLog(HibernateUtil.class);
    private static final SessionFactory sessionFactory;
    static {
        try {
            sessionFactory = new Configuration().configure().buildSessionFactory();
        } catch (HibernateException ex) {
            LOG.error(ex.getMessage(), ex);
            throw new RuntimeException("Exception building SessionFactory: " + ex.getMessage());
        }
    }
    private static final ThreadLocal session = new ThreadLocal();

    public static Session currentSession() {
        try {
            Session s = (Session) session.get();
            // Open a new Session, if this Thread has none yet
            if (s == null) {
                s = sessionFactory.openSession();
                session.set(s);
            }
            return s;
        } catch (HibernateException ex) {
            LOG.error(ex.getMessage(), ex);
            throw new RuntimeException("Exception in currentSession: " + ex.getMessage());
        }
    }

    public static void closeSession() {
        try {
            Session s = (Session) session.get();
            session.set(null);
            if (s != null) {
                s.close();
            }
        } catch (HibernateException ex) {
            LOG.error(ex.getMessage(), ex);
            throw new RuntimeException("Exception in closeSession: " + ex.getMessage());
        }
    }
    
    public static boolean isClosed() {
        return (session.get() == null);
    }
    
    public static void flush() {
        try {
            HibernateUtil.currentSession().flush();
        } catch (HibernateException ex) {
            LOG.error(ex.getMessage(), ex);
            throw new RuntimeException(ex.getMessage());
        }
    }
}
